/***********************************************************************
    Authors: Sparks (Gameplay Scripting) + Infinity Ward (Level Design)
    Map: Backlot
    Campaign: Arabian (Yeah, you play the bad guys, cool beans)
    Objectives:
    	1) Hold the city.
    	2) Retrieve classified documents.
    	3) Free fellow soldiers.
    	4) Grab sniper on roof.
    	5) Hold the city till reinforcements arrive.
    	6) Meet up with Al Asad.
    	
    Things to do/ keep in mind:
    	1) Tighten opening scene for more realistic feel
    		-- Something about the opening helicopters feels not CoD-ish
    		-- Make it seem like war is already going on
    			-- Doesn't feel right for level story if player shows up and the party starts
    		-- Add drones/actor models into the helis
    		-- Add RPGs into background for more realism
    		-- Player is "floating" up the stairs; fix or kill off, not professional
    	2) MG Nest in center route
    		-- Direct player to take side routes
    		-- Center route makes first objective too easy
    			-- Be sure friendlies guide the player to take another route
    	3) Friendly chatter for dynamic gameplay
    		-- Direct player more
    			-- Subtitles needed
    		-- Place emphasis on objectives
    			-- Subtitles needed
    		-- Make player aware of certain hot spot battles
    			-- Don't let player walk into a hot spot and get killed, isn't fair gameplay
    			-- Subtitles needed
    	4) Micromanage enemies less
    		-- Bigger goal radius
    		-- Assign color groups
    	5) Tighten the water tank sequence
    		-- Looks unprofessional at the moment
    			-- If guy dies early, level breaks
    	6) Tweak/Balance gameplay battle spots
    		-- Need more friendly spawner locations
    			-- Have sequences where player walks into area with friendlies already fighting
    				-- Adds more dynamic
    		-- Enemy spawners placed better
    			-- Player should see them run to positions more
    			-- Less spawning into cover a la Quake style
    			-- Don't have one trigger spawn a chunk of spawners
    				-- Spawn in increments from seperate triggers
    	7) Tweak/Tighten the objectives
    		-- Players need more visual detail for documents objective
    		-- Players are lost trying to get into center building
    			-- Add in ladders or stepping stones?
    			-- Add breach door?
    		-- Directions to get onto roof are vague
    		-- Make sure objectives can't be broken by player
    			-- Seal off center building till objective
    			-- Player can get the Barrett on roof before objective
    	8) Micromanage player less
    		-- Don't hold the player's hand so much
    		-- Let player decide how to proceed with objective
    		-- Make the player feel part of the war
    		-- Avoid "The player showed up, start the war"
    			-- Player is part of a bigger war, but important role nonetheless
***********************************************************************/

#include maps\_utility;
#include animscripts\utility;
#include common_scripts\utility;
#include maps\_anim;

main()
{
	/*--------------------
	 DON'T LOAD SCRIPT WHEN COMPILING REFLECTIONS
	----------------------*/
	if ( getdvar( "r_reflectionProbeGenerate" ) == "1" )
		return;
	
	/*--------------------
	 START POINTS
	----------------------*/
	// Handled by main so declare before _load
	add_start( "intro", ::start_intro, &"BACKLOT_START_INTRO" );
	add_start( "objective1", ::start_objective1, &"BACKLOT_START_OBJECTIVE1" );
	add_start( "objective2", ::start_objective2, &"BACKLOT_START_OBJECTIVE2" );
	add_start( "objective3", ::start_objective3, &"BACKLOT_START_OBJECTIVE3" );
	add_start( "objective4", ::start_objective4, &"BACKLOT_START_OBJECTIVE4" );
	add_start( "objective5", ::start_objective5, &"BACKLOT_START_OBJECTIVE5" );
	add_start( "credits", ::start_credits, &"BACKLOT_START_CREDITS" );
	default_start( ::start_intro );
	
	/*--------------------
	 PRECACHE
	----------------------*/
	precacheItem( "cobra_seeker" );
	precacheModel( "weapon_saw_MG_setup" );
	precachemodel("com_clipboard_wpaper_obj");
	precacheString(&"BACKLOT_INTROSCREEN_LINE_1");
	precacheString(&"BACKLOT_INTROSCREEN_LINE_2");
	precacheString(&"BACKLOT_INTROSCREEN_LINE_3");
	precacheString(&"BACKLOT_INTROSCREEN_LINE_4");
	precacheString(&"BACKLOT_INTROSCREEN_LINE_5");
	precacheString(&"BACKLOT_CREDIT1");
	precacheString(&"BACKLOT_CREDIT2");
	precacheString(&"BACKLOT_CREDIT3");
	precacheString(&"BACKLOT_OBJ1");
	precacheString(&"BACKLOT_OBJ2");
	precacheString(&"BACKLOT_OBJ3");
	precacheString(&"BACKLOT_OBJ4");
	precacheString(&"BACKLOT_OBJ5");
	precacheString(&"BACKLOT_OBJ6");
	precacheString(&"BACKLOT_GET_PLANS");
	precacheString(&"BACKLOT_REINFORCEMENTS");
	precacheshader("white");
	precacheshader("black");
	
	/*--------------------
	 SCRIPT VEHICLES
	----------------------*/
	level thread maps\_bradley::main( "vehicle_bradley" );
	level thread maps\_blackhawk::main( "vehicle_blackhawk" );
	level thread maps\_humvee::main( "vehicle_humvee_desert" );
	//level thread maps\_technical::main( "vehicle_pickup_technical" );
	level thread maps\_mi28::main( "vehicle_mi-28_flying" );
	level thread maps\_mi17::main( "vehicle_mi17_woodland_fly_cheap", undefined, true );
	
	/*--------------------
	 GLOGBAL SCRIPTS
	----------------------*/
	maps\_breach_hinges_left::main(); 
	maps\_breach::main(); 
	maps\_load::main();
	
	// Set Up Teams
	array_thread( getEntArray( "arabians", "script_noteworthy" ), ::add_spawn_function, ::setTeam, "allies" );
	array_thread( getEntArray( "americans", "script_noteworthy" ), ::add_spawn_function, ::setTeam, "axis" );
	
	// Fixes a minor bug IW shipped with affecting all custom SP maps
	level.early_level[ level.script ] = false;
	
	maps\_compass::setupMiniMap( "compass_map_mp_backlot" );
	setsaveddvar("compassmaxrange","1000");
	maps\backlot_anim::main();
	maps\backlot_amb::main();
	maps\backlot_fx::main();
	thread animscripts\dog_init::initDogAnimations();
	backlot_art();
	
	/*--------------------
	 WEAPON CLIPS
	----------------------*/
	level.weaponClipModels = [];
	level.weaponClipModels[0] = "weapon_dragunov_clip";
	level.weaponClipModels[1] = "weapon_ak47_clip";
	level.weaponClipModels[2] = "weapon_g3_clip";
	level.weaponClipModels[3] = "weapon_ak74u_clip";
	level.weaponClipModels[4] = "weapon_m16_clip";
	level.weaponClipModels[5] = "weapon_saw_clip";
	
	/*--------------------
	 CAMPAIGN
	----------------------*/
	level.campaign = "arab";
		
	/*--------------------
	 FLAGS
	----------------------*/
	flag_init( "begin_asad" );
	flag_init( "begin_documents" );
	flag_init( "begin_forces" );
	flag_init( "begin_roof_defend" );
	flag_init( "begin_sniper" );
	flag_init( "can_talk" );
	flag_init( "done_walking" );
	flag_init( "end_song_ready" );
	flag_init( "introscreen_done" );
	flag_init( "player_exit_intro" );
	flag_init( "rpg_ready" );
	flag_init( "heli_ready" );
	flag_init( "sniper_song_ready" );
	flag_init( "waves_finished" );
	flag_init( "wrong_way_no_more" );
	
	/*--------------------
	 HINTS
	----------------------*/
	
	/*--------------------
	 GLOBAL/MISC THREADS
	----------------------*/
	// Stuff that always need to be called, regardless of start points
	thread player_loadout();
	thread friendly_talk();
	thread ambient_explosions();
	thread objectives();
	thread music();
	
	// Debug Stuff
	//getEnt( "debug_this_trigger", "targetname" ) thread debug_me();
	
	// Non-Spawners
	array_thread( getAiArray( "axis" ), ::setTeam, "allies" );
	
	// MG Dude
	getEnt( "mg_dude", "script_noteworthy" ) thread mg_dude_think();
	
	// Bradley
	getEnt( "bradley_start", "targetname" ) thread manage_bradley();
	
	// Friendly Spawners
	level.MaxSparkys = 4;
	array_thread( getEntArray( "sparky_spawner", "targetname" ), ::sparky_spawners );
	
	// Friendly Chains
	array_thread( getEntArray( "sparky_chains", "targetname" ), ::sparky_chains );
	
	// Friendly Balancer
	//array_thread( getEntArray( "friendly_balancer", "targetname" ), ::friendly_balancer );
	
	// Flickering Light
	//getEnt( "flickering_light", "targetname" ) thread flicker_me();
	
	// Dog hack
	getEnt( "auto47", "targetname" ) thread add_spawn_function( ::doggie_name );
	
	// Background Helis
	thread maps\_vehicle::create_vehicle_from_spawngroup_and_gopath( 5 );
	thread maps\_vehicle::create_vehicle_from_spawngroup_and_gopath( 9 );
	thread maps\_vehicle::create_vehicle_from_spawngroup_and_gopath( 2 );
	thread maps\_vehicle::create_vehicle_from_spawngroup_and_gopath( 12 );
}

/***********************************************************************
    START FUNCTIONS
***********************************************************************/ 

start_intro()
{
	/*--------------------
	 OPENING OF MAP THREADED FROM HERE
	----------------------*/
	level.player setorigin( level.player.origin );
	level.player setplayerangles( level.player.angles );
	level.player freezeControls( true );
	level.player allowsprint( false );
	thread introscreen();
	thread opening_scene();
} // start_intro()

start_objective1()
{
	/*--------------------
	 SKIP INTRO CINEMATIC
	----------------------*/
	// Objectives
	flag_set( "begin_documents" );
	
	// Music
	flag_set( "player_exit_intro" );
	
	level.player setorigin( level.player.origin );
	level.player setplayerangles( level.player.angles );
	level.player freezeControls( false );
	level.player allowsprint( true );
} // start_objective1()

start_objective2()
{
	/*--------------------
	 START RESUE OBJECTIVE
	----------------------*/
	// Objectives
	flag_set( "begin_documents" );
	flag_set( "begin_forces" );
	
	// Music
	flag_set( "player_exit_intro" );
	
	level.player setorigin( ( 720, -432, 64 ) );
	level.player setplayerangles( ( 0, 270, 0 ) );
	level.player freezeControls( false );
	level.player allowsprint( true );
} // start_objective1()

start_objective3()
{
	/*--------------------
	 START DEFEND OBJECTIVE
	----------------------*/
	// Objectives
	flag_set( "begin_documents" );
	flag_set( "begin_forces" );
	flag_set( "begin_sniper" );
	
	// Music
	flag_set( "player_exit_intro" );
	
	level.player setorigin( ( 352, -320, 416 ) );
	level.player setplayerangles( ( 0, 210, 0 ) );
	level.player freezeControls( false );
	level.player allowsprint( true );
} // start_objective3()

start_objective4()
{
	/*--------------------
	 START DEFEND OBJECTIVE
	----------------------*/
	// Objectives
	flag_set( "begin_documents" );
	flag_set( "begin_forces" );
	flag_set( "begin_sniper" );
	flag_set( "begin_roof_defend" );
	
	// Music
	flag_set( "player_exit_intro" );
	flag_set( "sniper_song_ready" );
	
	level.player setorigin( ( 512, -288, 128 ) );
	level.player setplayerangles( ( 0, 180, 0 ) );
	level.player freezeControls( false );
	level.player allowsprint( true );
}

start_objective5()
{
	/*--------------------
	 MEET WITH AL ASAD // ENDING
	----------------------*/
	// Objectives
	flag_set( "begin_documents" );
	flag_set( "begin_forces" );
	flag_set( "begin_sniper" );
	flag_set( "begin_roof_defend" );
	flag_set( "begin_asad" );
	
	// Music
	flag_set( "player_exit_intro" );
	flag_set( "sniper_song_ready" );
	flag_set( "end_song_ready" );
	
	level.player setorigin( ( 1616, 720, 80 ) );
	level.player setplayerangles( ( 0, 2.50448, 0 ) );
	level.player freezeControls( false );
	level.player allowsprint( true );
}

start_credits()
{
	/*--------------------
	 TEST CREDITS
	----------------------*/
	level.player setorigin( level.player.origin );
	level.player setplayerangles( level.player.angles );
	
	thread credits();
} // start_credits()

/***********************************************************************
    LEVEL FUNCTIONS
***********************************************************************/ 

player_loadout()
{
	/*--------------------
	 GIVE THE PLAYER SOME WEAPONS
	----------------------*/
	level.player takeallweapons();
	level.player giveWeapon( "Beretta" );
	level.player giveWeapon( "ak47_grenadier" );
	level.player giveWeapon( "fraggrenade" );
	level.player giveWeapon( "flash_grenade" );
	level.player setOffhandSecondaryClass( "flash" );
	level.player switchToWeapon( "ak47_grenadier" );
	level.player setViewmodel( "viewhands_desert_opfor" );
} // player_loadout()

music()
{
	/*--------------------
	 HANDLES LEVEL TRACKS
	----------------------*/
	// Opening Song
	MusicPlay( "intro_song" );
	flag_wait( "player_exit_intro" );
	wait ( 0.2 );
	MusicStop( 0.4 );
	
	// Sniper Rooftop Mission
	flag_wait( "sniper_song_ready" );
	MusicPlay( "sniper_song" );
	
	// Closing Dialogue/Credits
	flag_wait( "end_song_ready" );
	MusicStop();
	MusicPlay( "end_song" );
} // music()

objectives()
{
	/*--------------------
	 OBJECTIVES LOGIC
	----------------------*/
	Objective_Add( 0, "active", &"BACKLOT_OBJ1" );
	doc_trig = getEnt( "documents_trig", "targetname" );
	assertEx( isDefined( doc_trig ), "Documents trigger does not exist" );
	
	flag_wait( "begin_documents" );
	Objective_Add( 1, "current", &"BACKLOT_OBJ2", ( 1499.7, 258.7, 287 ) );
	doc_trig thread documents_obj();
	delaythread( 1, ::autosave_by_name, "documents" );
	
	flag_wait( "begin_forces" );
	thread setup_second_obj();
	Objective_State( 1, "done" );
	Objective_Add( 2, "current", &"BACKLOT_OBJ3", ( 408, -392, 136 ) );
	
	flag_wait( "begin_sniper" );
	thread rooftop_obj();
	Objective_State( 2, "done" );
	Objective_Add( 3, "current", &"BACKLOT_OBJ4", ( 264, -352, 404 ) );
	
	//flag_wait( "begin_roof_defend" );
	//Objective_State( 3, "done" );
	//Objective_Add( 4, "current", &"BACKLOT_OBJ5" );
	delaythread( 1, ::autosave_by_name, "defend" );
	
	flag_wait( "begin_asad" );
	delaythread( 1, ::autosave_by_name, "asad" );
	Objective_State( 3, "done" );
	setup_main_characters();
	Objective_Add( 4, "current", &"BACKLOT_OBJ6", level.Asad getOrigin() );
	getEnt( "ending", "targetname" ) thread closing_scene();
} // objectives()

documents_obj()
{
	/*--------------------
	 SET UP DOCUMENTS
	----------------------*/
	documents = getEnt( "documents", "targetname" );
	assertEx( isDefined( documents ), "Documents do not exist" );
	
	obj = spawn("script_model", ( 1503.5, 267.5, 289.5 ));
	obj.angles = ( 0, 334.1, 0 );
	obj setmodel("com_clipboard_wpaper_obj");
	
	self SetHintString( &"BACKLOT_GET_PLANS" );
	self waittill ( "trigger" );
	thread play_sound_in_space( "intelligence_pickup_clipboard", obj.origin );
	
	self thread trigger_off();
	obj hide();
	documents hide();
	flag_set( "begin_forces" );
	
	getEnt( "use_windows", "targetname" ) thread window_hint();
} // documents_obj()

window_hint()
{
	/*--------------------
	 DIRECT PLAYER TO CLIMB INTO CARSHOP
	----------------------*/
	self waittill ( "trigger" );	
	guy = get_closest_ai( level.player getOrigin(), "axis" );
	assertEx( isDefined( guy ), "No AI available near player." );
	if( isDefined( guy ) )
	{
		guy.AnimName = "guy";
		guy thread anim_single_solo ( guy, "find_way_in" );	
	}
	
	thread breach_carshop();
} // window_hint()

breach_carshop()
{
	/*--------------------
	 DIRECT NEWBS TO WALK INTO CARSHOP
	----------------------*/
	aBreachers = [];
	bSpawners = getEntArray( "breacher_carshop", "targetname" );
	for( x = 0 ; x < bSpawners.size ; x++ )
	{
		aBreachers[ x ] = bSpawners[ x ] StalingradSpawn();
		aBreachers[ x ] thread magic_bullet_shield();
		aBreachers[ x ] setGoalNode( getNode( aBreachers[ x ].target, "targetname" ) );
	}
	
	aBreachers[ 0 ] waittill ( "goal" );
	eVolume = getEnt( "volume_breach_carshop", "targetname" );
	sBreachType = "shotgunhinges_breach_left";	
	eVolume thread maps\_breach::breach_think( aBreachers, sBreachType );
	
	wait ( 6 );
	for( x = 0 ; x < aBreachers.size ; x++ )
		aBreachers[ x ] thread stop_magic_bullet_shield();
} // breach_carshop()

setup_second_obj()
{
	/*--------------------
	 SET UP BUILDING IN CENTER
	----------------------*/
	windows = getEntArray( "center_building_windows", "targetname" );
	for( x = 0 ; x < windows.size ; x++ )
		windows[ x ] Delete();
		
	trig = getEnt( "center_building_guys", "targetname" );
	trig waittill ( "trigger" );
	
	enemies = getEntArray( trig.target, "targetname" );
	for( x = 0 ; x < enemies.size ; x++ )
		enemies[ x ] StalingradSpawn();
	trig Delete();
	
	// Door Kicker
	getEnt( "doorkick_trig", "targetname" ) thread door_kicker();
} // setup_second_obj()

rooftop_obj()
{
	/*--------------------
	 SET UP DEFEND OBJECTIVE (ORIGINALLY ROOFTOP)
	----------------------*/
	thread timer_director();
	
	while ( 1 )
	{
		//flag_wait( "begin_roof_defend" );
		flag_set( "sniper_song_ready" );
	
		level.RoofCounter = 0;
	
		spawners = getEntArray( "sniper_baddies", "targetname" );
		assertEx( isDefined( spawners ), "Sniper baddies do not exist." );
		array_thread( spawners, ::add_spawn_function, ::rooftop_obj_think );
	
		for( x = 0 ; x < ( ( RandomInt( spawners.size ) ) + 1 ) ; x++)
		{
			// We can only have 7 indices (AI) per objective
			if( level.RoofCounter < level.RoofMaxAi )
			{
				temp = spawners[ RandomInt( spawners.size ) ] doSpawn();
				if( spawn_failed( temp ) )
				{
					// Nothing
				}
				else
					level.RoofCounter++;
			}
			else
				wait ( 5 );
		}		
		wait ( 3 );
		
		if( flag( "waves_finished" ) )
			break;
	}
} // rooftop_sequence()

timer_director()
{
	/*--------------------
	 INITIALIZE TIMER
	----------------------*/
	//flag_set( "begin_roof_defend" );
	level.timer = maps\_hud_util::get_countdown_hud();
	level.timer.label = (&"BACKLOT_REINFORCEMENTS");
	level.timer settenthstimer( 90 );
	level.start_time = gettime();
	thread manage_defend_helis();
	thread roof_friendlies();
	
	wait ( 90 );
	flag_set( "waves_finished" );
	flag_set( "begin_asad" );
	level.timer destroy();
	
	thread reinforcements();
} // timer_director()

reinforcements()
{
	/*--------------------
	 MI28 FLIES IN AND TAKES OUT REMAINING BADDIES
	----------------------*/
	thread maps\_vehicle::create_vehicle_from_spawngroup_and_gopath( 18 );
	waittillframeend;
	target = getEnt( "heli_target_rein", "targetname" );
	heli = getEnt( "reinforcement_heli", "targetname" );
	
	patsys = getEntArray( "heli_patsys", "targetname" );
	for( x = 0 ; x < patsys.size ; x++ )
		patsys[ x ] StalingradSpawn();
	
	heli delaythread( 4.1, ::heli_shoots_rockets_at_ent, target );
	
	wait( 9 );
	heli thread chopper_ai_mode_aim_turret();
	heli thread chopper_ai_mode_shoot_turret();
	
	guy = get_closest_ai( level.player getOrigin(), "axis" );
	assertEx( isDefined( guy ), "No friendlies were alive." );
	if( isDefined( guy ) )
	{
		guy.animName = "guy";
		guy thread anim_single_solo( guy, "they_are_here" );
	}
} // reinforcements()

chopper_ai_mode_aim_turret()
{
	/*--------------------
	 MI28 AIMS AT ENEMIES
	----------------------*/
	level.cosine[ "45" ] = cos( 45 );
	self endon( "death" );	
	for(;;)
	{
		eTarget = maps\_helicopter_globals::getEnemyTarget( 6000, level.cosine[ "45" ], true, true, true, true );
		if( isdefined( eTarget ) )
		{
			eTargetOffset = ( 0, 0, 0 );
			self setTurretTargetEnt( eTarget, eTargetOffset );
		}
		wait randomfloatrange( 0.2, 1.0 );
	}
} // chopper_ai_mode_aim_turret()

chopper_ai_mode_shoot_turret()
{
	/*--------------------
	 MI28 FIRES AT ENEMIES
	----------------------*/
	self endon( "death" );
	self setVehWeapon( "hind_turret" );
	for(;;)
	{
		guys = getAiArray( "allies" );
		if( !isDefined( guys.size ) )
			continue;
		randomShots = randomintrange( 30, 60 );
		for( i = 0 ; i < randomShots ; i++ )
		{
			self fireWeapon( "tag_flash" );
			wait 0.05;
		}
		wait randomfloatrange( 1.0, 1.75 );
		guys = undefined;
	}
} // chopper_ai_mode_shoot_turret()

manage_bradley()
{
	self waittill ( "trigger" );
	wait ( 1.2 );
	bradley = getEnt( "bradley", "targetname" );
	bradley thread bradley_ai_mode_aim_turret();
	bradley thread bradley_ai_mode_shoot_turret();
	
	wait( 40 );
	if( isAlive( bradley ) )
		RadiusDamage( bradley.origin, 50, 1000, 100 );
} // manage_bradley()

bradley_ai_mode_aim_turret()
{
	/*--------------------
	 BRADLEY AIMS AT ENEMIES
	----------------------*/
	level.cosine[ "45" ] = cos( 45 );
	self endon( "death" );	
	for(;;)
	{
		eTarget = get_closest_ai( level.player getOrigin(), "allies" );
		if ( RandomInt( 4 ) == 2 ) 
			eTarget = level.player;
		
		if( isdefined( eTarget ) )
		{
			eTargetOffset = ( 8, 13, 10 );
			self setTurretTargetEnt( eTarget, eTargetOffset );
		}
		wait randomfloatrange( 1.5, 2.9 );
	}
} // bradley_ai_mode_aim_turret()

bradley_ai_mode_shoot_turret()
{
	/*--------------------
	 BRADLEY FIRES AT ENEMIES
	----------------------*/
	self endon( "death" );
	self setVehWeapon( "bradley_turret" );
	for(;;)
	{
		randomShots = randomintrange( 1, 2 );
		for( i = 0 ; i < randomShots ; i++ )
		{
			self fireWeapon( "tag_flash" );
			wait 0.05;
		}
		wait randomfloatrange( 1.0, 1.75 );
	}
} // bradley_ai_mode_shoot_turret()

rpg_think()
{
	self endon ( "death" );	
	wait( 1 );	
	helis = [];
	helis = getEntArray ( "heli_rpg_targets", "targetname" );
	self.drawoncompass = 0;
	self.pacifist = 1;
	self.ignoreme = 1;
	self set_ignoreSuppression( true );
	self thread rpg_delete();
	//self setstablemissile( true );
	
	for(;;)
	{
		eTarget = helis[ RandomInt( helis.size ) ];
		if( isDefined( eTarget ) )
		{
			self setentitytarget( eTarget );
			wait( 0.5 );
			self Shoot();
			wait( RandomIntRange( 5, 10 ) );
		}
		wait ( 0.01 );
	}
} // rpg_think()

rpg_delete()
{
	wait( RandomIntRange( 25, 40 ) );
	RadiusDamage( self getorigin(), 100, 1000, 900 );
} // rpg_delete()

manage_defend_helis()
{
	/*--------------------
	 IN-COMING WAVES OF ENEMIES
	----------------------*/
	// So heli doesn't crash into previous water tank
	//getEnt( "helicopter_crash_location_delete_me", "script_noteworthy" ) Delete();
	
	wait ( RandomIntRange( 0, 10 ) );
	thread maps\_vehicle::create_vehicle_from_spawngroup_and_gopath( 13 );
	
	wait ( RandomIntRange( 0, 20 ) );
	thread maps\_vehicle::create_vehicle_from_spawngroup_and_gopath( 15 );
	
	wait ( RandomIntRange( 0, 10 ) );
	thread maps\_vehicle::create_vehicle_from_spawngroup_and_gopath( 16 );
	
	wait ( RandomIntRange( 0, 10 ) );
	thread maps\_vehicle::create_vehicle_from_spawngroup_and_gopath( 17 );
	
	wait ( RandomIntRange( 0, 10 ) );
	thread maps\_vehicle::create_vehicle_from_spawngroup_and_gopath( 19 );
} // manage_defend_helis()

rooftop_obj_think()
{
	/*--------------------
	 HANDLES ROOFTOP TARGETS
	----------------------*/
	// During some instances RoofCounter may sometimes not be defined,
	// so just treat the spawned AI as a non-objective.
	if( isDefined( level.RoofCounter ) )
	{
		self.Counter = level.RoofCounter;
		//Objective_AdditionalPosition( 4, self.Counter, self.origin );
		while( isalive( self ) )
		{
			wait ( 0.2 );
			//Objective_AdditionalPosition( 4, self.Counter, self.origin );
		}
		self waittill ( "death" );
		level.RoofCounter--;
	}
}

roof_friendlies()
{
	/*--------------------
	 MANAGE FRIENDLIES IN CARSHOP
	----------------------*/
	spawners = getEntArray( "auto116", "targetname" );
	
	while( !flag( "waves_finished" ) )
	{
		guys = get_force_color_guys( "allies", "o" );
		
		num = 0;
		if( guys.size <= 2 )
		{
			num = RandomInt( 2 );
			if( ( !isDefined( num ) ) || ( num < 0 ) )
				continue;
				
			spawners[ num ] doSpawn();
		}
		wait( 3 );
	}
} // roof_friendlies()

setup_main_characters()
{	
	/*--------------------
	 SPAWN ENDING PEOPLE
	----------------------*/
	temp_a = getEnt( "asad", "targetname" );
	level.Asad = temp_a StalingradSpawn();
	if( spawn_failed( level.Asad ) )
	{
		assertMsg( "Asad did not spawn." );
	}
	level.Asad.team = "allies";
	level.Asad.ignoreme = true;
	level.Asad.pacifist = true;
	level.Asad.animName = "guy";
	level.Asad.goal_radius = 1;
			
	temp_v = getEnt( "vip", "targetname" );
	level.Vip = temp_v StalingradSpawn();
	if( spawn_failed( level.Vip ) )
	{
		assertMsg( "Vip did not spawn." );
	}
	level.Vip.team = "allies";
	level.Vip.ignoreme = true;
	level.Vip.pacifist = true;
	level.Vip.animName = "guy";
	level.Vip.goal_radius = 1;
	level.Vip gun_remove();
	
	temp_l = getEnt( "left_dragger", "targetname" );
	level.l_dragger = temp_l StalingradSpawn();
	if( spawn_failed( level.l_dragger ) )
	{
		assertMsg( "Left dragger did not spawn." );
	}
	level.l_dragger.team = "allies";
	level.l_dragger.ignoreme = true;
	level.l_dragger.pacifist = true;
	level.l_dragger.goal_radius = 1;
	level.l_dragger.animName = "guy";
	
	temp_r = getEnt( "right_dragger", "targetname" );
	level.r_dragger = temp_r StalingradSpawn();
	if( spawn_failed( level.r_dragger ) )
	{
		assertMsg( "Right dragger did not spawn." );
	}
	level.r_dragger.team = "allies";
	level.r_dragger.ignoreme = true;
	level.r_dragger.pacifist = true;
	level.r_dragger.goal_radius = 1;
	level.r_dragger.animName = "guy";
	
	thread maps\_vehicle::create_vehicle_from_spawngroup_and_gopath( 14 );
} // setup_main_characters()

closing_scene()
{
	/*--------------------
	 HANDLE CLOSING SCENE
	----------------------*/
	flag_set( "end_song_ready" );	
	spawners = undefined;
	spawners = [];
	spawners = getEntArray( "flood_and_secure", "targetname" );
	assertEx( isDefined( spawners ), "Closing scene - no flood and secures exist." );
	if( isDefined( spawners) )
		array_thread( spawners, ::trigger_off );
	spawners = undefined;
	spawners = getEntArray( "flood_spawner", "targetname" );
	assertEx( isDefined( spawners ), "Closing scene - no flood spawners exist." );
	if( isDefined( spawners) )
		array_thread( spawners, ::trigger_off );
	
	temp_v = getNode( "auto233", "targetname" );	
	temp_v thread anim_loop_solo( level.Vip, "cower_idle", undefined, "vip_stop" );
	
	//thread maps\_vehicle::create_vehicle_from_spawngroup_and_gopath( 15 );
	
	level.Vip.name = "Nikolai";
	level.Asad.name = "Al Asad";

	self waittill ( "trigger" );
	temp_v notify ( "vip_stop" );
	temp_v thread anim_single_solo( level.Asad, "zip_asad" );
	temp_v anim_single_solo( level.Vip, "zip_vip" );
	
	temp_v thread anim_loop_solo( level.Vip, "wounded_idle" );
	
	level.Asad thread set_generic_run_anim( "patrol_walk", true );
	level.Asad setGoalNode( getNode( "asad_speech", "targetname" ) );
	level.Asad.animName = "generic";
	l_node = getNode( level.l_dragger.target, "targetname" );
	r_node = getNode( level.r_dragger.target, "targetname" );
	l_node thread anim_loop_solo( level.l_dragger, "with_him" );
	r_node thread anim_loop_solo( level.r_dragger, "with_him_other_guy" );
	level.Asad anim_single_solo( level.Asad, "end_speech1" );
	wait ( .2 );
	
	guys  = get_force_color_guys( "allies", "p" );
	array_thread( guys, ::setup_crowd_anim );
	
	
	level.l_dragger anim_single_solo( level.l_dragger, "with_him_dia" );
	wait ( .4 );
	
	level.Asad anim_single_solo( level.Asad, "end_speech2" );
	wait( 2 );
	
	credits();
} // closing_scene()

setup_crowd_anim()
{
	/*--------------------
	 ASSIGNS GUN WAIVING ANIMS
	----------------------*/
	crowd_anim = [];
	crowd_anim[ 0 ] = "crowdmember_gunup_idle";
	crowd_anim[ 1 ] = "crowdmember_gunup_fire";
	crowd_anim[ 2 ] = "crowdmember_gundown_idle";
	crowd_anim[ 3 ] = "crowdmember_gundown_jeer";
	crowd_anim[ 4 ] = "crowdmember_gundown_fire_a";
	crowd_anim[ 5 ] = "crowdmember_gundown_fire_b";
	
	self.animName = "guy";
	self anim_loop_solo( self, crowd_anim[ RandomIntRange( 0, crowd_anim.size ) ] );
} // setup_crowd_anim()

introscreen()
{
	/*--------------------
	 TYPICAL INTROSCREEN
	----------------------*/
	level.intro_offset = ( -20 );
	lines = [];
	lines[ lines.size ] = &"BACKLOT_INTROSCREEN_LINE_1";
	lines[ "date" ] = &"BACKLOT_INTROSCREEN_LINE_2";
	lines[ lines.size ] = &"BACKLOT_INTROSCREEN_LINE_3";
	lines[ lines.size ] = &"BACKLOT_INTROSCREEN_LINE_5";
	maps\_introscreen::introscreen_feed_lines( lines );
	
	level.intro_offset = ( 0 );
	fade_time = ( 8 );
	time = ( 8.5 );
	maps\_introscreen::introscreen_generic_fade_in( "black", time, fade_time );
	
	flag_set( "introscreen_done" );
	level.player freezeControls( false );
	
	// Background RPGs
	rpg_guys = getEntArray( "rpg_intro_guys", "targetname" );
	//array_thread( rpg_guys, ::add_spawn_function, maps\_spawner::spawner_dronespawn, rpg_guys );
	array_thread( rpg_guys, ::rpg_think );
} // introscreen()

ambient_explosions()
{
	/*--------------------
	 EXPLOSIONS AROUND MAP
	----------------------*/
	expo_locs = [];
	expo_locs = getEntArray( "random_expo", "targetname" );
	assertEx( isDefined( expo_locs ), "Explosion locations do not exist." );
	
	level.ExpoSkill = 30; // Fail-safe
	thread ambient_explosions_think();
	
	while (1)
	{
		rand = RandomIntRange( 0, level.ExpoSkill );
		rand_loc = RandomIntRange( 0, expo_locs.size );
		if( (rand == 25) || (rand == 27) )
		{
			expo_locs[ rand_loc ] PlaySound( "explosion" );
			playFx( level._effect[ "explosion" ], expo_locs[ rand_loc ].origin );
			wait 0.2;
			Earthquake( 0.3, 3, level.player.origin, 500 );
		}
		else
			wait 1;
	}
} // ambient_explosions()

ambient_explosions_think()
{
	/*--------------------
	 BAESD ON GAMESKILL, CHANGE EXPLOSION OCCURANCE + MAX ROOFTOP BADDIES
	----------------------*/
	switch( level.gameSkill )
	{
		case 0: // Recruit -- Least amount of explosions/ baddies
			level.ExpoSkill = 23;
			level.RoofMaxAi = 10;
			break;
		case 1: // Easy
			level.ExpoSkill = 22;
			level.RoofMaxAi = 12;
			break;
		case 2: // Hardened
			level.ExpoSkill = 21;
			level.RoofMaxAi = 14;
			break;
		case 3: // Veteran -- Most amount of explosions/ baddies
			level.ExpoSkill = 19;
			level.RoofMaxAi = 15;
			break;
	}
} // ambient_explosions_think()

opening_scene()
{
	/*--------------------
	 OPENING SCENE, AFTER INTROSCREEN
	----------------------*/
	arab1 = getEnt( "arab1", "targetname" );
	arab1.animName = "arab1";
	arab1.goal_radius = 64;
	arab1.pacifist = 1;
	arab2 = getEnt( "arab2", "targetname" );
	arab2.animName = "arab2";
	arab2.goal_radius = 64;
	arab2.pacifist = 2;
	arab3 = getEnt( "arab3", "targetname" );
	arab3.animName = "arab3";
	arab3.goal_radius = 64;
	arab3.pacifist = 1;
	arab1 thread magic_bullet_shield();
	arab2 thread magic_bullet_shield();
	arab3 thread magic_bullet_shield();
	
	flag_wait( "introscreen_done" );
	
	white_overlay = create_overlay_element( "white", 1 );
	white_overlay fadeOverTime( 1 );
	white_overlay.alpha = 0;
		
	thread maps\_vehicle::create_vehicle_from_spawngroup_and_gopath( 11 );
	waittillframeend;
	// Water Tank
	getEnt( "tank_explosion", "targetname" ) thread water_tank_scene();
	
	wait ( 0.1 );
	
	/*--------------------
	 OPENING DIALOGUE
	----------------------*/
	// American's are attacking!
	arab1 thread anim_single_solo ( arab1, "intro" );
	
	level.player PlaySound( "explosion" );
	playFx( level._effect[ "explosion" ], ( 2664, 8, 408 ) );
	Earthquake( 0.3, 3, level.player.origin, 1000 );
	wait ( 0.8 );
	
	playFx( level._effect[ "explosion" ], ( 2200, -728, 264 ) );
	Earthquake( 0.1, 3, level.player.origin, 1000 );
	wait ( 0.2 );
	
	// We have to get out of here.
	arab2 thread anim_single_solo ( arab2, "intro" );
	wait ( 2 );
	
	// We have to secure military plans first!
	arab3 thread anim_single_solo ( arab3, "intro" );
	
	flag_set( "done_walking" );
	flag_set( "begin_documents" );
	level.player allowsprint( true );
	wait ( 8 );
		
	// Player hasn't moved, so AI lead player to move
	if ( level.player IsTouching( getEnt( "player_lets_go", "targetname" ) ) )
	{
		getEnt( "start1", "targetname" ) thread activate_trigger();
		
		guy = get_closest_ai( level.player getOrigin(), "axis" );
		assertEx( isDefined( guy ), "No AI available near player." );
		if( isDefined( guy ) )
		{
			guy.AnimName = "guy";
			guy thread anim_single_solo ( guy, "lets_go" );	
		}
	}
	
	arab1.pacifist = 0;
	arab2.pacifist = 0;
	arab3.pacifist = 0;
	arab1 thread stop_magic_bullet_shield();
	arab2 thread stop_magic_bullet_shield();
	arab3 thread stop_magic_bullet_shield();
	//thread rpg_guy();
} // opening_scene()

rpg_guy()
{
	/*--------------------
	 AMBIENT GUY SHOOTING HELI DOWN
	----------------------*/
	rpg_dude = getEnt( "rpg_dude", "targetname" );
	heli = getEnt( "heli_target", "targetname" );
	heli.health = 10;
	
	flag_wait( "heli_ready" );
	rpg_dude set_ignoreSuppression( true );
	rpg_dude setgoalnode( getnode( "rpg_dude_goal", "targetname" ) );
	rpg_dude setstablemissile( true );
	rpg_dude.goalradius = 64;
	
	flag_wait( "rpg_ready" );
	rpg_dude setentitytarget( heli );
	
	/*--------------------
	 HELI WAITTILLS FROM _VEHICLE UTILITY
	----------------------*/
	wait ( 8.0 );
	if( isDefined( rpg_dude ) && isAlive( rpg_dude ) )
	{
		rpg_dude ClearEntityTarget( heli );
		RadiusDamage( rpg_dude getorigin(), 100, 1000, 1 );
	}
} // rpg_guy()

credits()
{
	/*--------------------
	 PIGGY-BACKED THE INTROSCREEN, LOOKS BETTER THAN USING PRINT
	----------------------*/
	black_overlay = create_overlay_element( "black", 0 );
	black_overlay fadeOverTime( 3 );
	black_overlay.alpha = 1;	
	
	wait( 3 );
	black_overlay fadeOverTime( 0.001 );
	black_overlay.alpha = 0;	
	level.player freezeControls( true );
	MusicStop();
	//MusicPlayWrapper("credits_song");
	lines = [];
	lines[ lines.size ] = &"BACKLOT_CREDIT1"; // Sparks
	lines[ lines.size ] = &"BACKLOT_CREDIT2"; // Infinity Ward
	lines[ lines.size ] = &"BACKLOT_CREDIT3"; // Thanks For Playing
	maps\_introscreen::introscreen_feed_lines( lines );
	maps\_introscreen::introscreen_generic_fade_in( "black", 8.5, 8 );
	black_overlay fadeOverTime( 0.5 );
	black_overlay.alpha = 1;
		
	/*--------------------
	 QUIT MAP
	----------------------*/
	Changelevel( "" );
} // credits()

backlot_art()
{
	/*--------------------
	 IW STOCK ART SETTINGS
	----------------------*/
	level.tweakfile = true; 
	setExpFog( 901.718, 2200, 0.631373, 0.568627, 0.34902, 0 );
	VisionSetNaked( "mp_backlot", 0 );
} // backlot_art()

create_overlay_element( shader_name, start_alpha )
{
	/*--------------------
	 Jake Keating Tool
	----------------------*/
	overlay = newHudElem();
	overlay.x = 0;
	overlay.y = 0;
	overlay setshader ( shader_name, 640, 480);
	overlay.alignX = "left";
	overlay.alignY = "top";
	overlay.sort = 1;
	overlay.horzAlign = "fullscreen";
	overlay.vertAlign = "fullscreen";
	overlay.alpha = start_alpha;
	overlay.foreground = true;
	return overlay;
} // create_overlay_element()

setTeam( team )
{
	/*--------------------
	 CONFIGURES ARABIANS AS FRIENDLIES AND AMERICANS AS BADDIES
	----------------------*/
	self.team = team;
	if( team == "allies" )
	{
		self maps\_names::get_name_for_nationality( "arab" );
		self.goal_radius = 64;
	}
	else
	{
		// Couldn't figure out how to remove American names =0(
		// Null .name and .script_friendName cause switch statement errors in names utility
		// Not too important to spend time figuring out
		self.goal_radius  = 1024;
	}
} // setTeam()

door_kicker()
{
	/*--------------------
	 DOOR KICKER IN CENTER BUILDING
	----------------------*/
	door_d = getEnt( "door_down", "targetname" );
	door_d hide();
	
	self waittill ("trigger");
	spawner = getEnt( "doorkicker", "targetname" );
	guy = spawner doSpawn();
	waittillframeend;
	assertEx( isDefined( guy ), "Door kicker does not exist" );
	node = getNode ( guy.target, "targetname");
	guy.animname = "doorkicker";
	node thread anim_single_solo( guy, "kickdoor");
	
	wait ( 0.5 );
	door = getEnt ( "door", "targetname" );
	door playsound ( "kickdoor" );
	door MoveTo ( door_d getOrigin(), 0.1 );
	door RotatePitch( 90, 0.1 );
	door waittill( "movedone" );
	door ConnectPaths();
	
	/*--------------------
	 WARNS PLAYER OF ONCOMING WAVES
	----------------------*/
	wait ( 1 );
	guy = get_closest_ai( level.player getOrigin(), "axis" );
	assertEx( isDefined( guy ), "No AI available near player." );
	if( isDefined( guy ) )
	{
		guy.AnimName = "guy";
		guy thread anim_single_solo ( guy, "roofsniper" );	
	}
	
	guys = GetAIArray( "axis" );
	array_thread( guys, ::self_delete );
	
	guys = get_force_color_guys( "allies", "p" );
	array_thread( guys, ::self_delete );
	
	wait ( 1 );
	flag_set( "begin_sniper" );
} // door_kicker()

water_tank_scene()
{
	/*--------------------
	 TRIGGERED EXPLOSION SCENE OUTSIDE
	----------------------*/
	heli = getEnt( "heli_target", "targetname" );
	assertEx( isDefined( heli ), "Water tank helicopter doesn't exist." );
	heli.health = 10;
	RadiusDamage( heli getorigin(), 100, 1000, 900 );
	self waittill ("trigger");
	wait ( 1.9 );
	tank = getEnt( "water_tank", "targetname" );
	ladder = getEnt( "water_ladder", "targetname" );
	base = getEnt( "water_stand", "targetname" );	
	dude = getEnt( "water_tank_dude", "targetname" );
		
	if( isDefined( dude ) && isAlive( dude ) )
		RadiusDamage( dude getorigin(), 100, 1000, 1 );
	tank hide();
	ladder hide();
	wait ( 0.1 );
	
	/*--------------------
	 WATER FX
	----------------------*/
	expo_locs = [];
	expo_locs = getEntArray( "water_spot", "targetname" );
	
	for( x = 0 ; x < expo_locs.size ; x++)
	{
		playFx( level._effect[ "water" ], expo_locs[ x ] getorigin() );
		wait 0.01;
	}
} // water_tank_scene()

sparky_spawners()
{
	/*--------------------
	 FRIENDLY SPAWNERS
	----------------------*/
	while ( 1 )
	{
		self waittill( "trigger" );
		self thread trigger_off();
		sparky_wave_master( self );
		self thread trigger_on();
		wait( 1 );
	}
} // sparky_spawners()

sparky_wave_master(trigger)
{
	/*--------------------
	 HANDLE FRIENDLY REINFORCEMENTS
	----------------------*/
	guys = get_force_color_guys( "allies", "r" );
	spawn = getEntArray( trigger.target, "targetname" );
	num = 0;
	
	if ( guys.size < level.MaxSparkys )
	{
		num = RandomInt( spawn.size );
		if( isDefined( num ) )
				spawned = spawn[ num ] doSpawn();
		wait ( 3 );
	}
} // sparky_wave_master()

friendly_talk()
{
	/*--------------------
	 FRIENDLIES RANDOM CHAT ABOUT OBJECTIVES
	----------------------*/
	
	chatter = [];
	chatter = getEntArray( "friendly_chatter", "targetname" );
	assertEx( isDefined( chatter ), "Friendly chatter triggers do not exist." );
	
	for( x = 0 ; x < chatter.size ; x++ )
	{
		assertEx( isDefined( chatter[ x ].script_noteworthy ), "No track defined on trigger at: " + chatter[ x ].origin );
		if ( isDefined( chatter[ x ].script_noteworthy ) )
			chatter[ x ].Track = chatter[ x ].script_noteworthy;
		else
			chatter[ x ].Track = "null";
	}
	
	array_thread( chatter, ::friendly_talk_director );
} // friendly_talk()

friendly_talk_director()
{
	/*--------------------
	 MAKES THE FRIENDLY AI A LITTLE MORE DYNAMIC FOR GAMEPLAY
	----------------------*/
	self waittill ( "trigger" );
	guy = get_closest_ai( level.player getOrigin(), "axis" );
	assertEx( isDefined( guy ), "No AI available near player." );
	if( !isDefined( guy ) )
	{
		// There should NEVER be 0 friendlies alive in Backlot.
		// But if you want to use this for your map, script
		// something to fail-safe your map from erroring out.
		self thread trigger_off();
		return;
	}
	guy.AnimName = "guy";
	if( self.Track != "null" )
	{
		guy thread anim_single_solo ( guy, self.Track );
		self thread trigger_off();
	}
	else
	{
		self thread trigger_off();
	}
}

mg_dude_think()
{
	/*--------------------
	 MG SAW GUNNER
	----------------------*/
	while( !flag( "wrong_way_no_more" ) )
	{
		self waittill ( "trigger" );
		mg = getEnt( "mg_dude", "targetname" );
		assertEx( isDefined( mg ), "MG Gunner spawner does not exist." );
		dude = mg StalingradSpawn();
		if ( spawn_failed( dude ) )
		{
			assertMsg( "MG Gunner did not spawn." );
			return;
		}
		
		wait ( 1 );
		dude waittill ( "death" );
	}
	
	/*flag_wait( "wrong_way_no_more" );		
	wrong_way = [];
	wrong_way[ 0 ] = getEnt( "wrongway", "script_noteworthy" );
	wrong_way[ 1 ] = getEnt( "mg_dude", "script_noteworthy" );
	array_thread( wrong_way, ::trigger_off );*/
} // mg_dude_think()

friendly_balancer()
{
	/*--------------------
	 BALANCES AI // OBSOLETE NOW
	----------------------*/
	while( 1 )
	{
		self waittill ( "trigger" );
		thread friendly_balancer_manager( self );
		wait ( 2 );
	}
} // friendly_balancer()

friendly_balancer_manager(trigger)
{
	/*--------------------
	 DELETES AI IF NOT RELEVANT FOR GAMEPLAY // OBSOLETE NOW
	----------------------*/
	// Originally just for friendlies, but made the scope wider.
	// Having some errors with friendly (Arabian) color chains,
	// friendly AI won't lead/ follow player. So going with the
	// flow to create/micro-manage a chaotic atmosphere.
	
	//guys = get_force_color_guys( "allies", "r" );
	guys = undefined;
	guys = GetAIArray( "axis", "allies" );
	assertEx( isDefined( guys ), "Level balancer could not find any alive guys at all." );
	if( !isDefined( guys ) )
		return;
		
	for( x = 0 ; x < guys.size ; x++ )
	{
		assertEx( isDefined( guys[ x ].targetname ), "Guy at: " + ( guys[ x ] getOrigin() ) + " has no targetname." );
		if( guys[ x ] IsTouching( trigger ) )
		{
			//iPrintLn( "**Touching: " + ( guys[ x ] getOrigin() ) );
			// Don't delete
		}
		else
		{
			//iPrintLn( "**Deleted: " + ( guys[ x ] getOrigin() ) );
			RadiusDamage( guys[ x ] getorigin(), 25, 150, 100 );
			//guys[ x ] Delete();
		}
	}
	
	/*
	other_trigs = getEntArray( "friendly_balancer", "targetname" );
	assertEx( isDefined( other_trigs ), "Other triggers do not exist." );
	if( !isDefined( other_trigs ) )
		return;
	for( x = 0 ; x < other_trigs.size ; x++ )
	{
		if( other_trigs[ x ].script_noteworthy != trigger.script_noteworthy )
			other_trigs[ x ] trigger_on();
	}
	trigger trigger_off();
	*/
} // friendly_balancer_manager()

sparky_chains()
{
	/*--------------------
	 CUSTOM FRIENDLY CHAIN // OBSOLETE NOW
	----------------------*/
	self waittill ( "trigger" );
	
	nodes = getNodeArray( self.target, "targetname" );
	assertEx( isDefined( nodes ), "No friendly chain nodes at: " + ( self getOrigin() ) );
	if( !isDefined( nodes) )
		return;
		
	array_thread( nodes, ::nodes_claimed_false );
		
	guys = get_force_color_guys( "allies", "r" );
	assertEx( isDefined( guys ), "No friendlies were alive." );
	if( !isDefined( guys ) )
		return;
		
	assertEx( nodes.size  >= guys.size, "Not enough friendly chain nodes at: " + ( self getOrigin() ) );
	for( x = 0 ; x < nodes.size ; x++ )
	{
		if( !isDefined( guys[ x ] ) )
			return;
		
		if( nodes[ x ].Claimed == false )	
		{
			guys[ x ] setGoalNode( nodes[ x ] );
			nodes[ x ].Claimed = true;
		}
	}
} // sparky_chains()

nodes_claimed_false()
{
	self.Claimed = false;
} // nodes_claimed_false()

flicker_me()
{
	/*--------------------
	 LIGHT FLICKERING // OBSOLETE NOW
	----------------------*/
	new_intensity = 2;
	while( 1 )
	{
		new_intensity *= randomfloatrange( 0.3, 1.7 );
		self setLightIntensity( new_intensity );
		wait( 0.05 );
	}
} // nodes_claimed_false()

heli_shoots_rockets_at_ent( target )
{
	/*--------------------
	 MAKES MI28 SHOOT ROCKETS
	----------------------*/
	attractor = missile_createAttractorEnt( target, 100000, 60000 );
	self maps\_helicopter_globals::fire_missile( "mi28_seeker", 3, target, .75 );
	wait( 5 );
	missile_deleteAttractor( attractor );
} // heli_shoots_rockets_at_ent()

doggie_name()
{
	self.name = "Cuddles";
} // doggie_name()